﻿using System;
using System.Collections.Generic;
using System.Drawing;

namespace Event
{
    class Program
    {
        static void Main(string[] args)
        {
            GameUnit exampleUnit = new Soldier();

            BattleMapDisplay battleMap = new BattleMapDisplay(exampleUnit);
            OverviewDisplay overviewMap = new OverviewDisplay(exampleUnit);

            exampleUnit.Move(new Point(10, 10));
            exampleUnit.Move(new Point(20, 20));
        }
    }

    #region Strategy Pattern Stuff
    public interface IShootingBehaviour
    {
        void Shoot();
    }

    public class NormalShot : IShootingBehaviour
    {
        public void Shoot()
        {
            /* Shoot Logic */
        }
    }

    public class CantShoot : IShootingBehaviour
    {
        public void Shoot()
        {
            /* No Shoot Logic */
        }
    }

    public class ThrowKnives : IShootingBehaviour
    {
        public void Shoot()
        {
            /* Throwing Logic */
        }
    }
    #endregion

    public class PositionChangedEventArgs : EventArgs
    {
        public Point NewLocation { get; private set; }

        public PositionChangedEventArgs(Point newLocation)
        {
            NewLocation = newLocation;
        }
    }

    public abstract class GameUnit
    {
        public event EventHandler<PositionChangedEventArgs> PositionChanged;

        public Point Position { get; protected set; }
        public IShootingBehaviour ShootingBehaviour { get; protected set; }

        // Unit classes each take care of their own Display
        public abstract void Display();

        protected void OnPositionChanged()
        {
            if (PositionChanged != null)
            {
                PositionChanged(this, new PositionChangedEventArgs(Position));
            }
        }
       
        public void PerformShoot()
        {
            ShootingBehaviour.Shoot();
        }

        public void Move(Point newLocation)
        {
            Position = newLocation;
            OnPositionChanged();
        }
    }

    #region Units
    public class Soldier : GameUnit
    {
        public Soldier()
        {
            ShootingBehaviour = new NormalShot();
        }

        public override void Display()
        {
            /* Display code */
        }
    }

    public class Tank : GameUnit
    {
        public Tank()
        {
            ShootingBehaviour = new NormalShot();
        }

        public override void Display()
        {
            /* Display code */
        }
    }

    public class Engineer : GameUnit
    {
        public Engineer()
        {
            ShootingBehaviour = new CantShoot();
        }

        public override void Display()
        {
            /* Display code */
        }
    }

    public class Civilian : GameUnit
    {
        public Civilian()
        {
            ShootingBehaviour = new CantShoot();
        }

        public override void Display()
        {
            /* Display code */
        }
    }

    public class GangMember : GameUnit
    {
        public GangMember()
        {
            ShootingBehaviour = new ThrowKnives();
        }

        public override void Display()
        {
            /* Display code */
        }
    }
    #endregion

    #region Screens
    public class BattleMapDisplay
    {
        public BattleMapDisplay(GameUnit exampleUnit)
        {
            // In reality, you might have a singleton Game.Units somewhere, holding a collection
            // of GameUnits over which you can iterate and subscribe to. For this example
            // however, I'm simply showing how to subscribe.

            //GameUnit exampleUnit = new Soldier(); 
            exampleUnit.PositionChanged += new EventHandler<PositionChangedEventArgs>(exampleUnit_PositionChanged);
        }

        private void exampleUnit_PositionChanged(object sender, PositionChangedEventArgs e)
        {
            /* Update the location of the unit on the map */
            // e.NewLocation // Can also cast sender to GameUnit, but not strongly typed
            Console.WriteLine(string.Format("Battle Map - Changed, {0}", e.NewLocation));
        }
    }

    public class OverviewDisplay
    {
        public OverviewDisplay(GameUnit exampleUnit)
        {
            // In reality, you might have a singleton Game.Units somewhere, holding a collection
            // of GameUnits over which you can iterate and subscribe to. For this example
            // however, I'm simply showing how to subscribe.

            //GameUnit exampleUnit = new Soldier(); 
            exampleUnit.PositionChanged += new EventHandler<PositionChangedEventArgs>(exampleUnit_PositionChanged);
        }

        private void exampleUnit_PositionChanged(object sender, PositionChangedEventArgs e)
        {
            /* Update the location of the unit on the map */
            // e.NewLocation // Can also cast sender to GameUnit, but not strongly typed
            Console.WriteLine(string.Format("Overview Map - Changed, {0}", e.NewLocation));
        }
    }
    #endregion
}